==> xl-plugin-manager-25.1.0-102.113-sources.jar: com/xebialabs/plugin/manager/repository/sql/SqlPluginRepository.scala <==
package com.xebialabs.plugin.manager.repository.sql

import com.xebialabs.plugin.manager.metadata.Version
import com.xebialabs.plugin.manager.model.DbPlugin
import com.xebialabs.plugin.manager.rest.api.{PluginSource, PluginStatus}
import com.xebialabs.plugin.manager.repository.sql.MigrationsSchema.{XLPM_PLUGIN, XLPM_PLUGIN_BYTES}
import com.xebialabs.plugin.manager.repository.sql.SqlPluginRepository.UpdateCommand
import grizzled.slf4j.Logging
import org.springframework.jdbc.core.namedparam.{MapSqlParameterSource, NamedParameterJdbcTemplate}
import org.springframework.jdbc.core.support.SqlLobValue
import org.springframework.jdbc.core.{JdbcTemplate, RowMapper}
import org.springframework.jdbc.support.GeneratedKeyHolder
import org.springframework.util.StreamUtils.copyToByteArray

import java.io.ByteArrayInputStream
import java.sql.{Connection, ResultSet, Types}
import scala.jdk.CollectionConverters.{ListHasAsScala, MapHasAsJava}
import scala.util.{Failure, Success, Try, Using}

class SqlPluginRepository(jdbcTemplate: JdbcTemplate) extends PluginQueries with Logging {

  implicit lazy val namedTemplate: NamedParameterJdbcTemplate = new NamedParameterJdbcTemplate(jdbcTemplate)
  lazy val dialect: Dialect = Dialect(jdbcTemplate)

  def insert(plugin: DbPlugin): Unit = {
    val keyHolder = new GeneratedKeyHolder
    val mapSqlParameterSource = new MapSqlParameterSource()
    val bytes = plugin.bytes.get
    mapSqlParameterSource.addValue(XLPM_PLUGIN_BYTES.bytes, new SqlLobValue(new ByteArrayInputStream(bytes), bytes.length), Types.BLOB)
    val keyColumnNames = Array[String] {
      dialect.withDialect("id")
    } // leave lowercase "id" for compatibility with Postgres
    namedTemplate.update(INSERT_PLUGIN_BYTES, mapSqlParameterSource, keyHolder, keyColumnNames)

    namedTemplate.update(INSERT_PLUGIN, new MapSqlParameterSource(params(
      XLPM_PLUGIN.name -> plugin.name,
      XLPM_PLUGIN.version -> plugin.version.orNull,
      XLPM_PLUGIN.extension -> plugin.extension,
      XLPM_PLUGIN.groupId -> plugin.groupId,
      XLPM_PLUGIN.installationStatus -> plugin.installationStatus.toString,
      XLPM_PLUGIN.source -> plugin.source.toString,
      XLPM_PLUGIN.checksum -> plugin.checksum,
      XLPM_PLUGIN.pluginBytesId -> keyHolder.getKey.intValue).asJava))
  }

  def update(pluginFromApi: DbPlugin, pluginToUpdate: DbPlugin): Unit = {
    val mapSqlParameterSource = new MapSqlParameterSource()
    val bytes = pluginFromApi.bytes.get
    mapSqlParameterSource.addValue(XLPM_PLUGIN_BYTES.bytes, new SqlLobValue(new ByteArrayInputStream(bytes), bytes.length), Types.BLOB)
    mapSqlParameterSource.addValue(XLPM_PLUGIN_BYTES.id, pluginToUpdate.pluginBytesId.get, Types.BIGINT)
    namedTemplate.update(UPDATE_PLUGIN_BYTES, mapSqlParameterSource)

    namedTemplate.update(UPDATE_PLUGIN, new MapSqlParameterSource(params(
      XLPM_PLUGIN.version -> pluginFromApi.version.orNull,
      XLPM_PLUGIN.groupId -> pluginFromApi.groupId,
      XLPM_PLUGIN.installationStatus -> pluginFromApi.installationStatus.toString,
      XLPM_PLUGIN.source -> pluginFromApi.source.toString,
      XLPM_PLUGIN.checksum -> pluginFromApi.checksum,
      XLPM_PLUGIN.id -> pluginToUpdate.id).asJava))
  }

  def delete(plugin: DbPlugin): Int = {
    namedTemplate.update(DELETE_PLUGIN, new MapSqlParameterSource(params(
      XLPM_PLUGIN.id -> plugin.id).asJava))
    namedTemplate.update(DELETE_PLUGIN_BYTES, new MapSqlParameterSource(params(
      XLPM_PLUGIN_BYTES.id -> plugin.pluginBytesId.get).asJava))
  }

  def getByName(pluginName: String): Seq[DbPlugin] = {
    Try(
      namedTemplate.query(SELECT_BY_NAME, params(XLPM_PLUGIN.name -> pluginName).asJava, dbPluginMapper).asScala.toSeq
    ).getOrElse(Seq.empty[DbPlugin])
  }

  def getByNameAndVersion(name: String, version: Option[Version]): Seq[DbPlugin] = {
    val parameters = new MapSqlParameterSource(params(XLPM_PLUGIN.name -> name).asJava)

    val query = version match {
      case Some(v) =>
        parameters.addValue(XLPM_PLUGIN.version, v.id)
        SELECT_BY_NAME_AND_VERSION
      case None => SELECT_BY_NAME
    }

    Try(
      namedTemplate.query(query, parameters, dbPluginMapper).asScala.toSeq
    ).getOrElse(Seq.empty[DbPlugin])
  }

  def pluginTablesExist: Boolean = {
    Using(namedTemplate.getJdbcTemplate.getDataSource.getConnection()) { connection =>
      doesTableExist(connection, XLPM_PLUGIN.TABLE) || doesTableExist(connection, XLPM_PLUGIN.TABLE.toLowerCase())
    }.get
  }

  def doesTableExist(connection: Connection, tableName: String): Boolean = {
    val tables = connection.getMetaData.getTables(null, null, tableName, null)
    tables.next()
  }

  def getAllWithBytes: Seq[DbPlugin] = {
    val plugins = namedTemplate.query(SELECT_WITH_BYTES, dbPluginMapper)
    debug(s"Found the following plugins in the database: $plugins")
    plugins.asScala.toSeq
  }

  def getAllWithoutBytes: Seq[DbPlugin] = {
    val plugins = namedTemplate.query(SELECT, dbPluginWithoutBytesMapper)
    debug(s"Found the following plugins in the database: $plugins")
    plugins.asScala.toSeq
  }

  def updateAllPluginsStatusTo(status: PluginStatus.Value): Int = {
    namedTemplate.update(UPDATE_ALL_PLUGINS_STATUS, new MapSqlParameterSource("STATUS", status.toString))
  }

  def update(cmd: UpdateCommand): Unit = {
    cmd match {
      case SqlPluginRepository.UpdatePluginStatus(pluginName, newStatus) =>
        updatePluginStatus(pluginName, newStatus)
    }
  }


  private def updatePluginStatus(pluginName: String, newStatus: PluginStatus.Value) = {
    namedTemplate.update(UPDATE_PLUGIN_STATUS,
      params(
        XLPM_PLUGIN.name -> pluginName,
        XLPM_PLUGIN.installationStatus -> newStatus.toString
      ).asJava
    )
  }


  def getPluginBy(repositoryId: PluginSource.Value, groupId: String, artifactId: String, version: Option[String]): Option[DbPlugin] = {
    val parameters = new MapSqlParameterSource(params(
      XLPM_PLUGIN.source -> repositoryId.toString,
      XLPM_PLUGIN.groupId -> groupId,
      XLPM_PLUGIN.name -> artifactId).asJava)
    if (version.isDefined) parameters.addValue(XLPM_PLUGIN.version, version.get)

    val query = if (version.isDefined) {
      SELECT_BY_REPO_GROUPID_ARTIFACTID_VERSION_STATUS
    } else {
      SELECT_BY_REPO_GROUPID_ARTIFACTID_STATUS
    }

    Try(namedTemplate.queryForObject(query, parameters, dbPluginWithoutBytesMapper)) match {
      case Failure(_) => None
      case Success(plugin) => Some(plugin)
    }
  }

  private def dbPluginMapper: RowMapper[DbPlugin] = (rs: ResultSet, _: Int) => {
    val bytesInputStream = rs.getBinaryStream(XLPM_PLUGIN_BYTES.bytes)
    Try(copyToByteArray(bytesInputStream)) match {
      case Success(bytes) => DbPlugin(
        rs.getLong(XLPM_PLUGIN.id),
        rs.getString(XLPM_PLUGIN.name),
        Option(rs.getString(XLPM_PLUGIN.version)),
        rs.getString(XLPM_PLUGIN.extension),
        rs.getString(XLPM_PLUGIN.groupId),
        PluginStatus.withName(rs.getString(XLPM_PLUGIN.installationStatus)),
        PluginSource.withName(rs.getString(XLPM_PLUGIN.source)),
        rs.getString(XLPM_PLUGIN.checksum),
        Some(rs.getLong(XLPM_PLUGIN.pluginBytesId)),
        Some(bytes))
      case Failure(ex) => throw new IllegalStateException("Failed to read plugin from the db", ex)
    }
  }

  private def dbPluginWithoutBytesMapper: RowMapper[DbPlugin] = (rs: ResultSet, _: Int) =>
    DbPlugin(
      rs.getLong(XLPM_PLUGIN.id),
      rs.getString(XLPM_PLUGIN.name),
      Option(rs.getString(XLPM_PLUGIN.version)),
      rs.getString(XLPM_PLUGIN.extension),
      rs.getString(XLPM_PLUGIN.groupId),
      PluginStatus.withName(rs.getString(XLPM_PLUGIN.installationStatus)),
      PluginSource.withName(rs.getString(XLPM_PLUGIN.source)),
      rs.getString(XLPM_PLUGIN.checksum),
      Some(rs.getLong(XLPM_PLUGIN.pluginBytesId)),
      None)

  def params(map: (String, _ <: Any)*): Map[String, Any] = map.toMap[String, Any]
}

object SqlPluginRepository {

  sealed trait UpdateCommand

  case class UpdatePluginStatus(pluginName: String, newStatus: PluginStatus.Value) extends UpdateCommand
}

==> xl-plugin-manager-25.3.0-416.930-sources.jar: com/xebialabs/plugin/manager/repository/sql/SqlPluginRepository.scala <==
package com.xebialabs.plugin.manager.repository.sql

import com.xebialabs.plugin.manager.metadata.Version
import com.xebialabs.plugin.manager.model.DbPlugin
import com.xebialabs.plugin.manager.rest.api.{PluginSource, PluginStatus}
import com.xebialabs.plugin.manager.repository.sql.MigrationsSchema.{XLPM_PLUGIN, XLPM_PLUGIN_BYTES}
import com.xebialabs.plugin.manager.repository.sql.SqlPluginRepository.UpdateCommand
import grizzled.slf4j.Logging
import org.springframework.jdbc.core.namedparam.{MapSqlParameterSource, NamedParameterJdbcTemplate}
import org.springframework.jdbc.core.support.SqlLobValue
import org.springframework.jdbc.core.{JdbcTemplate, RowMapper}
import org.springframework.util.StreamUtils.copyToByteArray

import java.io.ByteArrayInputStream
import java.sql.{Connection, ResultSet, Types}
import scala.jdk.CollectionConverters.{ListHasAsScala, MapHasAsJava}
import scala.util.{Failure, Success, Try, Using}

class SqlPluginRepository(jdbcTemplate: JdbcTemplate, idProvider: () => Long) extends PluginQueries with Logging {

  implicit lazy val namedTemplate: NamedParameterJdbcTemplate = new NamedParameterJdbcTemplate(jdbcTemplate)
  lazy val dialect: Dialect = Dialect(jdbcTemplate)

  def insert(plugin: DbPlugin): Unit = {
    val mapSqlParameterSource = new MapSqlParameterSource()
    val bytes = plugin.bytes.get
    val bytesId: Long = idProvider()
    mapSqlParameterSource.addValue(XLPM_PLUGIN_BYTES.bytes, new SqlLobValue(new ByteArrayInputStream(bytes), bytes.length), Types.BLOB)
    mapSqlParameterSource.addValue(XLPM_PLUGIN_BYTES.id, bytesId)
    namedTemplate.update(INSERT_PLUGIN_BYTES, mapSqlParameterSource)

    val pluginId = idProvider()
    namedTemplate.update(INSERT_PLUGIN, new MapSqlParameterSource(params(
      XLPM_PLUGIN.id -> pluginId,
      XLPM_PLUGIN.name -> plugin.name,
      XLPM_PLUGIN.version -> plugin.version.orNull,
      XLPM_PLUGIN.extension -> plugin.extension,
      XLPM_PLUGIN.groupId -> plugin.groupId,
      XLPM_PLUGIN.installationStatus -> plugin.installationStatus.toString,
      XLPM_PLUGIN.source -> plugin.source.toString,
      XLPM_PLUGIN.checksum -> plugin.checksum,
      XLPM_PLUGIN.pluginBytesId -> bytesId).asJava))
  }

  def update(pluginFromApi: DbPlugin, pluginToUpdate: DbPlugin): Unit = {
    val mapSqlParameterSource = new MapSqlParameterSource()
    val bytes = pluginFromApi.bytes.get
    mapSqlParameterSource.addValue(XLPM_PLUGIN_BYTES.bytes, new SqlLobValue(new ByteArrayInputStream(bytes), bytes.length), Types.BLOB)
    mapSqlParameterSource.addValue(XLPM_PLUGIN_BYTES.id, pluginToUpdate.pluginBytesId.get, Types.BIGINT)
    namedTemplate.update(UPDATE_PLUGIN_BYTES, mapSqlParameterSource)

    namedTemplate.update(UPDATE_PLUGIN, new MapSqlParameterSource(params(
      XLPM_PLUGIN.version -> pluginFromApi.version.orNull,
      XLPM_PLUGIN.groupId -> pluginFromApi.groupId,
      XLPM_PLUGIN.installationStatus -> pluginFromApi.installationStatus.toString,
      XLPM_PLUGIN.source -> pluginFromApi.source.toString,
      XLPM_PLUGIN.checksum -> pluginFromApi.checksum,
      XLPM_PLUGIN.id -> pluginToUpdate.id).asJava))
  }

  def delete(plugin: DbPlugin): Int = {
    namedTemplate.update(DELETE_PLUGIN, new MapSqlParameterSource(params(
      XLPM_PLUGIN.id -> plugin.id).asJava))
    namedTemplate.update(DELETE_PLUGIN_BYTES, new MapSqlParameterSource(params(
      XLPM_PLUGIN_BYTES.id -> plugin.pluginBytesId.get).asJava))
  }

  def getByName(pluginName: String): Seq[DbPlugin] = {
    Try(
      namedTemplate.query(SELECT_BY_NAME, params(XLPM_PLUGIN.name -> pluginName).asJava, dbPluginMapper).asScala.toSeq
    ).getOrElse(Seq.empty[DbPlugin])
  }

  def getByNameAndVersion(name: String, version: Option[Version]): Seq[DbPlugin] = {
    val parameters = new MapSqlParameterSource(params(XLPM_PLUGIN.name -> name).asJava)

    val query = version match {
      case Some(v) =>
        parameters.addValue(XLPM_PLUGIN.version, v.id)
        SELECT_BY_NAME_AND_VERSION
      case None => SELECT_BY_NAME
    }

    Try(
      namedTemplate.query(query, parameters, dbPluginMapper).asScala.toSeq
    ).getOrElse(Seq.empty[DbPlugin])
  }

  def pluginTablesExist: Boolean = {
    Using(namedTemplate.getJdbcTemplate.getDataSource.getConnection()) { connection =>
      doesTableExist(connection, XLPM_PLUGIN.TABLE) || doesTableExist(connection, XLPM_PLUGIN.TABLE.toLowerCase())
    }.get
  }

  def doesTableExist(connection: Connection, tableName: String): Boolean = {
    val tables = connection.getMetaData.getTables(null, null, tableName, null)
    tables.next()
  }

  def getAllWithBytes: Seq[DbPlugin] = {
    val plugins = namedTemplate.query(SELECT_WITH_BYTES, dbPluginMapper)
    debug(s"Found the following plugins in the database: $plugins")
    plugins.asScala.toSeq
  }

  def getAllWithoutBytes: Seq[DbPlugin] = {
    val plugins = namedTemplate.query(SELECT, dbPluginWithoutBytesMapper)
    debug(s"Found the following plugins in the database: $plugins")
    plugins.asScala.toSeq
  }

  def updateAllPluginsStatusTo(status: PluginStatus.Value): Int = {
    namedTemplate.update(UPDATE_ALL_PLUGINS_STATUS, new MapSqlParameterSource("STATUS", status.toString))
  }

  def update(cmd: UpdateCommand): Unit = {
    cmd match {
      case SqlPluginRepository.UpdatePluginStatus(pluginName, newStatus) =>
        updatePluginStatus(pluginName, newStatus)
    }
  }


  private def updatePluginStatus(pluginName: String, newStatus: PluginStatus.Value) = {
    namedTemplate.update(UPDATE_PLUGIN_STATUS,
      params(
        XLPM_PLUGIN.name -> pluginName,
        XLPM_PLUGIN.installationStatus -> newStatus.toString
      ).asJava
    )
  }


  def getPluginBy(repositoryId: PluginSource.Value, groupId: String, artifactId: String, version: Option[String]): Option[DbPlugin] = {
    val parameters = new MapSqlParameterSource(params(
      XLPM_PLUGIN.source -> repositoryId.toString,
      XLPM_PLUGIN.groupId -> groupId,
      XLPM_PLUGIN.name -> artifactId).asJava)
    if (version.isDefined) parameters.addValue(XLPM_PLUGIN.version, version.get)

    val query = if (version.isDefined) {
      SELECT_BY_REPO_GROUPID_ARTIFACTID_VERSION_STATUS
    } else {
      SELECT_BY_REPO_GROUPID_ARTIFACTID_STATUS
    }

    Try(namedTemplate.queryForObject(query, parameters, dbPluginWithoutBytesMapper)) match {
      case Failure(_) => None
      case Success(plugin) => Some(plugin)
    }
  }

  private def dbPluginMapper: RowMapper[DbPlugin] = (rs: ResultSet, _: Int) => {
    val bytesInputStream = rs.getBinaryStream(XLPM_PLUGIN_BYTES.bytes)
    Try(copyToByteArray(bytesInputStream)) match {
      case Success(bytes) => DbPlugin(
        rs.getLong(XLPM_PLUGIN.id),
        rs.getString(XLPM_PLUGIN.name),
        Option(rs.getString(XLPM_PLUGIN.version)),
        rs.getString(XLPM_PLUGIN.extension),
        rs.getString(XLPM_PLUGIN.groupId),
        PluginStatus.withName(rs.getString(XLPM_PLUGIN.installationStatus)),
        PluginSource.withName(rs.getString(XLPM_PLUGIN.source)),
        rs.getString(XLPM_PLUGIN.checksum),
        Some(rs.getLong(XLPM_PLUGIN.pluginBytesId)),
        Some(bytes))
      case Failure(ex) => throw new IllegalStateException("Failed to read plugin from the db", ex)
    }
  }

  private def dbPluginWithoutBytesMapper: RowMapper[DbPlugin] = (rs: ResultSet, _: Int) =>
    DbPlugin(
      rs.getLong(XLPM_PLUGIN.id),
      rs.getString(XLPM_PLUGIN.name),
      Option(rs.getString(XLPM_PLUGIN.version)),
      rs.getString(XLPM_PLUGIN.extension),
      rs.getString(XLPM_PLUGIN.groupId),
      PluginStatus.withName(rs.getString(XLPM_PLUGIN.installationStatus)),
      PluginSource.withName(rs.getString(XLPM_PLUGIN.source)),
      rs.getString(XLPM_PLUGIN.checksum),
      Some(rs.getLong(XLPM_PLUGIN.pluginBytesId)),
      None)

  def params(map: (String, _ <: Any)*): Map[String, Any] = map.toMap[String, Any]
}

object SqlPluginRepository {

  sealed trait UpdateCommand

  case class UpdatePluginStatus(pluginName: String, newStatus: PluginStatus.Value) extends UpdateCommand
}
